"""OpenAPI explorer for Azure"""
from __future__ import annotations

import itertools
import json
from collections import defaultdict
from pathlib import Path
from typing import Any


class Reader:
	"""Read Microsoft OpenAPI specifications"""

	def __init__(self, openapi: dict):
		self.doc = openapi

	@classmethod
	def load(cls, fp) -> Reader:
		"""Load from a path or file-like object"""
		if isinstance(fp, (str, Path)):
			with open(fp, mode="r", encoding="utf-8") as fp:
				return Reader(json.load(fp))
		else:
			return Reader(json.load(fp))

	@property
	def paths(self):
		"""Get API paths (standard and ms xtended)"""
		return list(itertools.chain(self.doc["paths"].items(), self.doc.get("x-ms-paths", {}).items()))


def operations(path_object: dict):
	"""Extract operations from an OpenAPI Path object"""
	return {k: v for k, v in path_object.items() if k in {"get", "put", "post", "delete", "options", "head", "patch", "trace"}}


if __name__ == "__main__":
	import sys

	reader = Reader.load(sys.argv[1])

	out: dict[str, dict[str, Any]] = defaultdict(dict)
	for path, pathobj in reader.paths:
		for method, operationobj in operations(pathobj).items():
			out[path][method] = {k: operationobj[k] for k in ("description", "operationId")}

	print(json.dumps(out))



"""Access the Azure HTTP API"""
from __future__ import annotations

from typing import Any

import requests
from pydantic import BaseModel


class AzureError(Exception):
	"""An Azure-specific error"""

	def __init__(self, json):
		self.json = json


class AzRest:
	"""Access the Azure HTTP API"""

	def __init__(self, token, session: requests.Session, base_url: str = "https://management.azure.com"):
		self.token = token
		self.session = session

		self.base_url = base_url

	@classmethod
	def from_credential(cls, credential) -> AzRest:
		"""Create from an Azure credential"""
		token = credential.get_token("https://management.azure.com//.default")
		session = requests.Session()
		return cls(token, session)

	def call(self, req: requests.Request) -> Any:
		"""Make the request to Azure"""
		req.headers["Authorization"] = f"Bearer {self.token.token}"  # TODO: push down into self.session
		res = self.session.send(req.prepare())  # TODO: write yet another fun interface to Azure
		if not res.ok:
			raise AzureError(res.json())

		return res

	def get(self, slug: str, apiv: str) -> Any:
		"""GET request"""
		return self.call(requests.Request("GET", self.base_url + slug, params={"api-version": apiv})).json()

	def delete(self, slug: str, apiv: str) -> Any:
		"""DELETE request"""
		return self.call(requests.Request("DELETE", self.base_url + slug, params={"api-version": apiv}))

	def put(self, slug: str, apiv: str, body: BaseModel) -> Any:
		"""PUT request, serialising the body"""
		return self.call(
			requests.Request("PUT", self.base_url + slug, params={"api-version": apiv}, data=body.model_dump_json(), headers={"Content-Type": "application/json"})
		).json()




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: llamazure/azrest:llamazure.rid

from setuptools import setup

setup(**{
    'author': 'Daniel Goldman',
    'classifiers': [
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Utilities',
        'Topic :: Internet :: Log Analysis',
    ],
    'description': 'Azure REST client',
    'install_requires': (
        'pydantic>=2.0',
        'requests<3,>=2',
        'types-requests',
    ),
    'license': 'Round Robin 2.0.0',
    'long_description': """# llamazure.azrest : A REST client for the Azure API

Don't use this yet it's not good.
""",
    'long_description_content_type': 'text/markdown',
    'name': 'llamazure.azrest',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'llamazure',
        'llamazure.azrest',
    ),
    'python_requires': '>=3.8',
    'version': '0.0.2',
})
